# 
from vcCommand import *
import vcHelpers.Selection as selection
import vcMatrix, vcVector
import os, sys, os.path, csv
import math
import subprocess
import time
import sys
import xml.etree.cElementTree as ET
import export_from_vc
#import xml.dom.minidom
blender_exe_uri = r'"C:\Program Files\Blender Foundation\Blender\blender.exe"'

DEG2RAD = math.radians(1)
cmd = getCommand()
app = getApplication()
app2 = getApplication() #two app objects for two separate OnRender events
camera = app.findCamera()

sim = getSimulation()
scale = 0.01
rec_file = False

def sanitizefilename(fname):
  suspiciouscharacters = [r'/', '\\' ,'#',':',';','!','¤','%','&','?','^','*',' ','\t']
  for c in suspiciouscharacters:
    fname = fname.replace(c,'_')
  return fname


def getProp(ptype, pname, pval, steps = [], limits = [], visible = True, overwrite = False):
  owner = cmd
  p = owner.getProperty(pname)
  if p and overwrite:
    owner.deleteProperty(p)
  elif p:
    return p
  if steps:
    p = owner.createProperty(ptype,pname,VC_PROPERTY_STEP)
    p.StepValues = steps
  elif limits:
    p = owner.createProperty(ptype,pname,VC_PROPERTY_LIMIT)
    p.MinLimits = limits[0]
    p.MaxLimits = limits[1]
  else:
    p = owner.createProperty(ptype,pname)
  if ptype != VC_BUTTON:
    p.Value = pval
  p.IsVisible = visible
  return p



def resChanged(arg):
  if resProp.Value == '<Custom>':
    resXProp.WritableWhenConnected = True
    resXProp.WritableWhenDisconnected = True
    resYProp.WritableWhenConnected = True
    resYProp.WritableWhenDisconnected = True
  else:
    resXProp.WritableWhenConnected = False
    resXProp.WritableWhenDisconnected = False
    resYProp.WritableWhenConnected = False
    resYProp.WritableWhenDisconnected = False
    vals = [int(x) for x in resProp.Value.split('x')]
    resXProp.Value = vals[0]
    resYProp.Value = vals[1]


def createXML(uri, framecount):
  settings_dict = {}
  settings_dict['engine'] = 'CYCLES'
  res = [resXProp.Value, resYProp.Value]
  portrait = portraitProp.Value == 'Portrait'
  res = res[::-1] if portrait else res
  settings_dict['resolution_x'] = res[0]
  settings_dict['resolution_y'] = res[1]
  settings_dict['samples'] = samplesProp.Value
  settings_dict['filepath'] = str(filepathProp.Value).replace(r'file:///','')
  #
  settings_dict['resolution_percentage'] = 100
  settings_dict['openinblender'] = openinblenderProp.Value
  settings_dict['sample_clamp_direct'] = sampleclampdirectProp.Value
  settings_dict['sample_clamp_indirect'] = sampleclampindirectProp.Value
  settings_dict['transparent_max_bounces'] = transparentmax_bouncesProp.Value
  settings_dict['transparent_min_bounces'] = transparentmin_bouncesProp.Value
  settings_dict['max_bounces'] = maxbouncesProp.Value
  settings_dict['min_bounces'] = minbouncesProp.Value
  settings_dict['diffuse_bounces'] = diffusebouncesProp.Value
  settings_dict['glossy_bounces'] = glossybouncesProp.Value
  settings_dict['transmission_bounces'] = transmissionbouncesProp.Value
  settings_dict['volume_bounces'] = volumebouncesProp.Value
  settings_dict['aobounces'] = aobouncesProp.Value
  settings_dict['caustics_reflective'] = causticsProp.Value
  settings_dict['caustics_refractive'] = causticsProp.Value
  settings_dict['denoising'] = denoisingProp.Value
  settings_dict['tile_x'] = tilexProp.Value
  settings_dict['tile_y'] = tileyProp.Value
  settings_dict['frame_start'] = 0
  settings_dict['frame_end'] = framecount
  if framecount:
    print 'Animation rendering: %i frames' % framecount
  settings_dict['CameraFOV'] = fovProp.Value
  settings_dict['device'] = deviceProp.Value
  settings_dict['use_freestyle'] = freestyleProp.Value
  settings_dict['depthOFradius'] = depthOFradiusProp.Value*scale
  settings_dict['volumescatter'] = volumescatterProp.Value
  if depthOFnodeProp.Value:
    node = depthOFnodeProp.Value
    if node.Type == VC_COMPONENT:
      node_name = node.Name
      node_name = sanitizefilename(node_name)
    else:
      node_name = '%s__%s'  % (node.Component.Name, node.Name)
      node_name = sanitizefilename(node_name)
  else:
    node_name = ''
  settings_dict['depthOFnode'] = node_name
  #
  settings_dict['envimage'] = envimageProp.Value.replace('file:///','')
  settings_dict['envstrength'] = envstrengthProp.Value
  settings_dict['envtype'] = envtypeProp.Value
  rgb_vec = envcolorProp.Value
  settings_dict['envcolor'] = '%s;%s;%s' % (rgb_vec.X, rgb_vec.Y, rgb_vec.Z)
  settings_dict['envvisible'] = envvisibleProp.Value
  #
  settings_dict['floorvisible'] = floorvisibleProp.Value
  settings_dict['floorsize'] = floorsizeProp.Value
  settings_dict['floortype'] = floortypeProp.Value
  settings_dict['floorcolorimage'] = floorcolorimageProp.Value.replace(r'file:///','')
  settings_dict['floorbumpimage'] = floorbumpimageProp.Value.replace(r'file:///','')
  settings_dict['floorcolor'] = floorcolorProp.Value
  settings_dict['floortexturesize'] = floortexturesizeProp.Value*scale
  settings_dict['floor_twoD_bound'] = str([x*scale for x in twoD_bound])
  
  root = ET.Element("RenderSettings")
  for key, item in settings_dict.iteritems():
    ET.SubElement(root, key).text = str(item)
  tree = ET.ElementTree(root)
  tree.write(uri)

def parentVisible(n):
  if not n.Visible:
    return False
  if n.Parent and not parentVisible(n.Parent):
    return False
  return True


def hasgeometry(n):
  hasgeometry = False
  if not parentVisible(n):
    return False
  for gs in n.Geometry.GeometrySets:
    if gs.Type == VC_TRIANGLESET:
      if gs.PointCount > 0:
        hasgeometry = True
        return True



def exportType(x):
  if x.Type in [VC_NODE]:
    return 'NODE'
  type_prop = x.getProperty('BLENDER_OBJECT_TYPE')
  if type_prop:
    return type_prop.Value
  else:
    return 'NODE'


def findAllNodes():
  nodes = []
  for c in app.Components:
    nodes.append(c)
    kids = [x for x in c.Children if x.Type == VC_NODE]
    while kids:
      k = kids.pop(0)
      nodes.append(k)
      kids.extend([x for x in k.Children if x.Type == VC_NODE])
  return nodes


def setCustomDataProps(material_name, custom_data_val):
  m = app.findMaterial(material_name)
  if m:
    p = m.createProperty(VC_STRING, 'CustomData')
    p.Value = custom_data_val
  return m

def customizessomematerials():
  try:
    m = setCustomDataProps('orange', 'NOISE/0.2/500.0')
    m.Clearcoat = 0.6
    m.Roughness = 0.0
  except:
    pass
  try:
    m = setCustomDataProps('black', 'NOISE/0.3/400.0')
    m.Clearcoat = 0.2
    m.Roughness = 0.5
  except:
    pass  
  try:
    m = setCustomDataProps('aluminium', 'NOISE/0.1/800.0')
    m.Clearcoat = 0.0
    m.Roughness = 0.01
    m.Metallic = 1.0
  except:
    pass


def setBound(node,twoD_bound):
  bd = node.BoundDiagonal
  bc = node.BoundCenter
  V = vcVector.new
  wp = node.WorldPositionMatrix
  corners = [V(bd.X,bd.Y,0), V(bd.X,-bd.Y,0), V(-bd.X,-bd.Y,0), V(-bd.X,bd.Y,0)]
  corners = [wp * (bc+x) for x in corners]
  minX = min([x.X for x in corners])
  maxX = max([x.X for x in corners])
  minY = min([x.Y for x in corners])
  maxY = max([x.Y for x in corners])
  if minX < twoD_bound[0]:
    twoD_bound[0] = minX
  if maxX > twoD_bound[1]:
    twoD_bound[1] = maxX
  if minY < twoD_bound[2]:
    twoD_bound[2] = minY
  if maxY > twoD_bound[3]:
    twoD_bound[3] = maxY



LINECHANGE = '\n'
def exportGeo():
  global twoD_bound
  materials = {}
  material_file = None
  uri_root = cmd.TEMP_URI #r'C:\Users\AhoneSa1\Documents\Blender\VC_output'
  for root, folders, files in os.walk(uri_root):
    for f in files:
      thisuri = os.path.join( root, f )
      os.remove(thisuri)
  nodes = findAllNodes()
  all_nodes = [[x, exportType(x)] for x in nodes if hasgeometry(x)]
  twoD_bound = [9999999,-9999999,9999999,-9999999]
  material_table_csv = None
  material_data_dict = {}
  if materialTableProp.Value == 'Write To Table':
    thefolder, commandfilename = os.path.split(getCommandPath()[8:])
    material_table_csv_uri = os.path.join(thefolder, 'materials.csv')
    try:
      material_table_csv = open(material_table_csv_uri, 'w')
      SEP = tableSeparatorProp.Value
      material_table_csv.write(SEP.join(material_columns)+LINECHANGE)
    except:
      material_table_csv = None
      print 'Unable to open table file for writing "%s"' % material_table_csv_uri
      print 'Make sure it is not open in another software.'
  elif materialTableProp.Value == 'Read From Table':
    thefolder, commandfilename = os.path.split(getCommandPath()[8:])
    material_table_csv_uri = os.path.join(thefolder, 'materials.csv')
    with open(material_table_csv_uri, 'r') as material_table_csv:
      SEP = tableSeparatorProp.Value
      tablereader = csv.reader(material_table_csv, delimiter=SEP)
      tablereader.next() #skip header
      for csvline in tablereader:
        csvline = list(csvline)
        material_data_dict[csvline[0]] = {}
        for title, csv_value in zip(material_columns, csvline)[1:]:
          material_data_dict[csvline[0]][title] = csv_value
  # loop layout content
  for node, object_type in all_nodes:
    if object_type in ['RECORDER', 'POINT_LIGHT', 'AREA_LIGHT', 'SUN_LIGHT', 'SPOT_LIGHT']:
      # skip abstract recorder components
      continue
    if node.Type in [VC_COMPONENT, VC_NODE]:
      setBound(node,twoD_bound)
    if node.Type == VC_COMPONENT:
      #comp name
      node_name = node.Name
    else:
      #comp name + node name
      node_name = '%s__%s'  % (node.Component.Name, node.Name)
    node_name = sanitizefilename(node_name)
    fileuri = os.path.join(uri_root, node_name+'.obj')
    SEP = tableSeparatorProp.Value
    ok, materials, material_file = export_from_vc.export_obj(app, node, materials, material_file, scale, fileuri, materialTableProp.Value, material_table_csv, material_data_dict, SEP)
  if material_table_csv:
    material_table_csv.close()
  overshoot = floorsizeProp.Value
  twoD_bound[0] -= overshoot
  twoD_bound[1] += overshoot
  twoD_bound[2] -= overshoot
  twoD_bound[3] += overshoot
  camera = app.findCamera()
  all_nodes.insert(0,[camera, 'CAMERA'])
  return all_nodes



def recKeyFrame(rec_data):
  global all_nodes, a
  if not rec_data:
    all_nodes = exportGeo()
    a=0 # first key frame
  sim.update()
  for i, token in enumerate(all_nodes):
    item, item_type = token
    deadComponent = False
    try:
      item.Name #this fails if a component has disappeared from the layout
    except:
      deadComponent = True
    if item_type == 'CAMERA':
      # element is camera
      object_name = 'VC_TO_BLENDER_CAMERA_OBJECT'
      mtx = camera.Matrix
      rot = mtx.WPR
      eye = camera.Eye
      coi = camera.Coi
      position_data = eye.X*scale, eye.Y*scale, eye.Z*scale, coi.X*scale, coi.Y*scale, coi.Z*scale
      if a == 0:
        object_data = []
    elif item_type in ['AREA_LIGHT', 'POINT_LIGHT', 'SUN_LIGHT', 'SPOT_LIGHT']:
      object_name = item.Name
      object_name = sanitizefilename(object_name)
      rgb = [item.R, item.G, item.B]
      strength = item.Strength
      if a == 0 and item_type in ['AREA_LIGHT']:
        sizeX = item.SizeX*scale
        sizeY = item.SizeY*scale
        falloff = item.Falloff
        object_data = [rgb, strength, sizeX, sizeY, falloff]
      elif a == 0 and item_type in ['POINT_LIGHT']:
        size = item.Size*scale
        falloff = item.Falloff
        object_data = [rgb, strength, size, falloff]
      elif a == 0 and item_type in ['SUN_LIGHT']:
        size = 0.1
        object_data = [rgb, strength, size]
      elif a == 0 and item_type in ['SPOT_LIGHT']:
        size = item.Size*scale
        angle = item.Angle
        falloff = item.Falloff
        object_data = [rgb, strength, size, angle, falloff]
      p = item.WorldPositionMatrix.P
      coi = vcMatrix.new(item.WorldPositionMatrix)
      coi.translateRel(0,0,-1000)
      coi = coi.P
      position_data = p.X*scale, p.Y*scale, p.Z*scale, coi.X*scale, coi.Y*scale, coi.Z*scale
    elif item_type == 'NODE':
      if deadComponent:
        position_data = rec_data[i][3][-1] #last keyframe
        # scale to zero so that the node disappears
        position_data[-1] = 0
        position_data[-2] = 0
        position_data[-3] = 0
      else:
        if item.Type == VC_COMPONENT:
          object_name = item.Name
        else:
          object_name = '%s__%s'  % (item.Component.Name, item.Name)
        object_name = sanitizefilename(object_name)
        mtx = item.WorldPositionMatrix
        rot = mtx.WPR
        sx = mtx.N.length()
        sy = mtx.O.length()
        sz = mtx.A.length()
        position_data = [mtx.P.X*scale, mtx.P.Y*scale, mtx.P.Z*scale, rot.X*DEG2RAD, rot.Y*DEG2RAD, rot.Z*DEG2RAD, sx,sy,sz]
        if a == 0:
          object_data = []
    if a == 0:
      rec_data.append( [object_name, item_type, object_data, [position_data]] )
    else:
      rec_data[i][3].append(position_data)
  return rec_data


'''
def sanitizefilename(fname):
  #print 'pre',fname
  suspiciouscharacters = ['#',':',';','!','¤','%','&','?','^','*',' ','\t']
  for c in suspiciouscharacters:
    fname = fname.replace(c,'_')
  #print 'post',fname
  return fname'''

def CallStill(arg):
  showBorderProp.Value = False
  rec_data = []
  recKeyFrame(rec_data)
  CallBlender(rec_data)

def CallAnim(arg):
  global rec_data
  showBorderProp.Value = False
  try:
    CallBlender(rec_data)
  except:
    print '*** Error in animation rendering call. ***'
    print ' - Make sure you first check "RecAnimation" check box'
    print ' - Then run simulation once until "RecAnimation" is automatically unchecked'
    print ' - Then hit "RenderAnimation"'

def OnSimRender(sim):
  global rec_data, a
  if showBorderProp.Value: showBorderProp.Value = False
  t = sim.SimTime
  if startanim_prop.Value > endanim_prop.Value:
    app.messageBox('Error: Animation start time is higher than end time.', 'Animation recording error', VC_MESSAGE_TYPE_ERROR, VC_MESSAGE_BUTTONS_OK)
    app.Simulation.reset()
  if t > startanim_prop.Value and t < endanim_prop.Value:
    rec_data = recKeyFrame(rec_data)
    a += 1
  elif t > endanim_prop.Value:
    app.OnRender = None
    recanim_prop.Value = False

def InitAnim(arg):
  global rec_data, a
  if recanim_prop.Value:
    rec_data = []
    app.RealTimeMode = False
    app.Simulation.SimSpeed = stepanim_prop.Value
    app.OnRender = OnSimRender
    a = 0
  else:
    app.OnRender = None
  #showBorderProp.Value = False

def CallBlender(rec_data):
  uri_root = cmd.TEMP_URI
  rec_file_name = os.path.join(uri_root, 'recording.vcbx')
  with open(rec_file_name, 'w') as rec_file:
    rec_file.write( str(rec_data) )
  xml_uri = os.path.join(uri_root, 'render_config.xml')
  createXML(xml_uri, a)
  thefolder, commandfilename = os.path.split(getCommandPath()[8:])
  vcimporteruri = os.path.join(thefolder, 'import_to_blender.py')
  cli_data = ''
  cli_data += blender_exe_uri
  if not openinblenderProp.Value:
    cli_data += ' --background'
  cli_data += ' --python'
  cli_data += r' "%s"' % vcimporteruri
  cli_data += ' --'
  cli_data += ' "%s"' % uri_root
  subprocess.Popen(cli_data)

def addLamp(arg):
  thefolder, commandfilename = os.path.split(getCommandPath()[8:])
  lamp = None
  if 'Sun' in arg.Name:
    lampuri = os.path.join('file///:',thefolder, 'Lights', 'BlenderSunLight.vcmx')
    lamp_strength = lightStrengthProp.Value/10000.0
  if 'Spot' in arg.Name:
    lampuri = os.path.join('file///:',thefolder, 'Lights', 'BlenderSpotLight.vcmx')
    lamp_strength = lightStrengthProp.Value
  if 'Area' in arg.Name:
    lampuri = os.path.join('file///:',thefolder, 'Lights', 'BlenderAreaLight.vcmx')
    lamp_strength = lightStrengthProp.Value
  elif 'Point' in arg.Name:
    lampuri = os.path.join('file///:',thefolder, 'Lights', 'BlenderPointLight.vcmx')
    lamp_strength = lightStrengthProp.Value
  lamp = app.load(lampuri)
  c = camera.Coi
  e = camera.Eye
  m = vcMatrix.new()
  m.P = e
  v = e-c
  v.normalize()
  m.setIJK(v.X, v.Y, v.Z)
  lamp.PositionMatrix = m
  lamp.Strength = lamp_strength
  lamp.update()
  app.render()


def OnStart():
  reload( export_from_vc )
  geocont = sim.World.UserGeometry
  geocont.clear()
  app.render()
  #
  # DEFAULT TAB
  global filepathProp, tempuriProp, samplesProp, resProp, resXProp, resYProp, portraitProp, renderstill_prop
  thefolder, commandfilename = os.path.split(getCommandPath()[8:])
  renderuri = os.path.join('file///:', thefolder, 'Output', 'Picture.png')
  filepathProp = getProp(VC_URI, 'Output', str(renderuri), visible = True, overwrite = False)
  #filepathProp = getProp(VC_URI, 'Output', renderuri.encode('ascii','replace'), visible = True, overwrite = False)
  thefolder, commandfilename = os.path.split(getCommandPath()[8:])
  tempuri = os.path.join('file///:', thefolder, 'TEMPDATA')
  tempuriProp = getProp(VC_URI, 'TEMP_URI', str(tempuri), visible = False, overwrite = False)
  samplesProp = getProp(VC_INTEGER, 'Samples', 100, visible = True, overwrite = False)
  #
  res_list = ['<Custom>', '300x300', '384x216', '640x360', '640x640' ,'1280x720', '1920x1080', '3840x2160']
  resProp = getProp(VC_STRING, 'Resolution', '<Custom>', steps = res_list, limits = [], visible = True, overwrite = False)
  resXProp = getProp(VC_INTEGER, 'ResX', 384, visible = True, overwrite = False)
  resYProp = getProp(VC_INTEGER, 'ResY', 216, visible = True, overwrite = False)
  resXProp.OnChanged = showBorderChangedEvent
  resYProp.OnChanged = showBorderChangedEvent
  portraitProp = getProp(VC_STRING, 'Page', 'Landscape', steps = ['Landscape', 'Portrait'], visible = True, overwrite = False)
  portraitProp.OnChanged = fovChangedEvent
  resProp.OnChanged = resChanged
  renderstill_prop = getProp(VC_BUTTON, 'Render', True, visible = True, overwrite = False)
  renderstill_prop.OnChanged = CallStill
  # 
  # LIGHTS TAB
  global addPointLightProp, addAreaLightProp, addSpotLightProp, addSunLightProp, lightStrengthProp
  addPointLightProp = getProp(VC_BUTTON, 'Lights::Add Point Light', False, visible = True, overwrite = False)
  addPointLightProp.OnChanged = addLamp
  addAreaLightProp = getProp(VC_BUTTON, 'Lights::Add Area Light', False, visible = True, overwrite = False)
  addAreaLightProp.OnChanged = addLamp
  addSpotLightProp = getProp(VC_BUTTON, 'Lights::Add Spot Light', False, visible = True, overwrite = False)
  addSpotLightProp.OnChanged = addLamp
  addSunLightProp = getProp(VC_BUTTON, 'Lights::Add Sun Light', False, visible = True, overwrite = False)
  addSunLightProp.OnChanged = addLamp
  lightStrengthProp = getProp(VC_REAL, 'Lights::Default Strength', 15000, visible = True, overwrite = False)
  #
  # ADVANCED TAB
  global causticsProp, freestyleProp, fovProp, showBorderProp, openinblenderProp, deviceProp, sampleclampdirectProp, sampleclampindirectProp
  global transparentmax_bouncesProp, transparentmin_bouncesProp, maxbouncesProp, minbouncesProp, diffusebouncesProp
  global glossybouncesProp, transmissionbouncesProp, volumebouncesProp, aobouncesProp, denoisingProp, tilexProp, tileyProp, materialTableProp, tableSeparatorProp
  global depthOFnodeProp, depthOFradiusProp, volumescatterProp, cameraProp, viewProp
  causticsProp = getProp(VC_BOOLEAN, 'Advanced::Caustics', False, visible = True, overwrite = False)
  freestyleProp = getProp(VC_BOOLEAN, 'Advanced::UseFreestyle', False, visible = True, overwrite = False)
  fovProp = getProp(VC_REAL, 'Advanced::CameraFovX', 45.0, visible = True, overwrite = False)
  showBorderProp = getProp(VC_BOOLEAN, 'Advanced::ShowBorder', False, visible = True, overwrite = False)
  fovProp.OnChanged = fovChangedEvent
  showBorderProp.OnChanged = showBorderChangedEvent
  openinblenderProp = getProp(VC_BOOLEAN, 'Advanced::OpenInBlender', False, visible = True, overwrite = False)
  deviceProp = getProp(VC_STRING, 'Advanced::RenderDevice', 'GPU', steps = ['CPU', 'GPU'], visible = True, overwrite = False)
  sampleclampdirectProp = getProp(VC_INTEGER, 'Advanced::SampleClampDirect', 2, visible = True, overwrite = False)
  sampleclampindirectProp = getProp(VC_INTEGER, 'Advanced::SampleClampIndirect', 2, visible = True, overwrite = False)
  transparentmax_bouncesProp = getProp(VC_INTEGER, 'Advanced::TransparentMaxBounces', 2, visible = True, overwrite = False)
  transparentmin_bouncesProp = getProp(VC_INTEGER, 'Advanced::TransparentMinBounces', 0, visible = True, overwrite = False)
  maxbouncesProp = getProp(VC_INTEGER, 'Advanced::MaxBounces', 4, visible = True, overwrite = False)
  minbouncesProp = getProp(VC_INTEGER, 'Advanced::MinBounces', 0, visible = True, overwrite = False)
  diffusebouncesProp = getProp(VC_INTEGER, 'Advanced::DiffuseBounces', 2, visible = True, overwrite = False)
  glossybouncesProp = getProp(VC_INTEGER, 'Advanced::GlossyBounces', 2, visible = True, overwrite = False)
  transmissionbouncesProp = getProp(VC_INTEGER, 'Advanced::TransmissionBounces', 8, visible = True, overwrite = False)
  volumebouncesProp = getProp(VC_INTEGER, 'Advanced::VolumeBounces', 0, visible = True, overwrite = False)
  aobouncesProp = getProp(VC_INTEGER, 'Advanced::AOBounces', 0, visible = True, overwrite = False)
  denoisingProp = getProp(VC_REAL, 'Advanced::Denoising', 0.5, visible = True, overwrite = False)
  tilexProp = getProp(VC_INTEGER, 'Advanced::TileX', 256, visible = True, overwrite = False)
  tileyProp = getProp(VC_INTEGER, 'Advanced::TileY', 256, visible = True, overwrite = False)
  materialTableProp = getProp(VC_STRING, 'Advanced::Materials', 'Default', steps = ['Default','Read From Table', 'Write To Table'], visible = True, overwrite = False)
  tableSeparatorProp = getProp(VC_STRING, 'Advanced::TableDelimiter', ';', steps = [';', ','], visible = True, overwrite = False)
  depthOFnodeProp = getProp('Ref<Node>', 'Advanced::DepthOfFieldNode', None, visible = True, overwrite = False)
  depthOFradiusProp = getProp(VC_REAL, 'Advanced::DepthOfFieldRadius', 50, visible = True, overwrite = False) 
  volumescatterProp = getProp(VC_REAL, 'Advanced::Mist', 0.0, visible = True, overwrite = False)
  

  #
  # ANIMATION TAB
  global recanim_prop, startanim_prop, endanim_prop, stepanim_prop, renderanim_prop
  recanim_prop = getProp(VC_BOOLEAN, 'Animation::RecAnimation', False, visible = True, overwrite = True)
  recanim_prop.OnChanged = InitAnim
  startanim_prop = getProp(VC_REAL, 'Animation::Start', 0.0, visible = True, overwrite = True)
  endanim_prop = getProp(VC_REAL, 'Animation::End', 1.0, visible = True, overwrite = True)
  stepanim_prop = getProp(VC_REAL, 'Animation::StepSize', 1.0/25.0, visible = True, overwrite = True)
  stepanim_prop.OnChanged = InitAnim
  renderanim_prop = getProp(VC_BUTTON, 'Animation::RenderAnimation', True, visible = True, overwrite = True)
  renderanim_prop.OnChanged = CallAnim
  #
  # ENV TAB
  global envtypeProp, envimageProp, envcolorProp, envstrengthProp, envvisibleProp
  thefolder, commandfilename = os.path.split(getCommandPath()[8:])
  envmapuri = os.path.join('file:///'+thefolder, 'Lights', 'Evn_room.bmp')
  envtypeProp = getProp(VC_STRING, 'Env::Background', 'Image', steps = ['Image', 'ColorRGB'], visible = True, overwrite = False)
  envtypeProp.OnChanged = envtypeChanged
  envimageProp = getProp(VC_URI, 'Env::Image', str(envmapuri), visible = True, overwrite = False)
  envcolorProp = getProp(VC_VECTOR, 'Env::ColorRGB', vcVector.new(1,1,1), visible = False, overwrite = False)
  envstrengthProp = getProp(VC_REAL, 'Env::Strength', 0.3, visible = True, overwrite = False)
  envvisibleProp = getProp(VC_BOOLEAN, 'Env::Visible', True, visible = True, overwrite = False)
  #
  # FLOOR TAB
  global floorvisibleProp, floorsizeProp, floortypeProp, floorcolorimageProp, floorbumpimageProp, floortexturesizeProp, floorcolorProp
  floorvisibleProp = getProp(VC_BOOLEAN, 'Floor::Visible', True, visible = True, overwrite = False)
  floorsizeProp = getProp(VC_REAL, 'Floor::BorderSize', 2000, visible = True, overwrite = False)
  floortypes_list = ['ConcreteLight', 'ConcreteDark', 'ConcreteWhite', 'SlateBrown', 'SlateGrey', 'SlateLight', 'Oak', 'Asphalt', 'Alu']
  floortypeProp = getProp(VC_STRING, 'Floor::Style', 'ConcreteLight', steps = floortypes_list, visible = True, overwrite = False)
  floorcolorimageProp = getProp(VC_URI, 'Floor::ColorTexture', '', visible = True, overwrite = False)
  floorbumpimageProp = getProp(VC_URI, 'Floor::BumpTexture', '', visible = True, overwrite = False)
  floortexturesizeProp = getProp(VC_REAL, 'Floor::TextureSize', 1200, visible = True, overwrite = False)
  floorcolorProp = getProp(VC_VECTOR, 'Floor::ColorRGB', vcVector.new(1,1,1), visible = False, overwrite = False)
  floortypeChanged(floortypeProp)
  floortypeProp.OnChanged = floortypeChanged
  #
  executeInActionPanel()
  #


def fovChangedEvent(arg):
  portrait = portraitProp.Value == 'Portrait'
  xRes = resYProp.Value if portrait else resXProp.Value
  yRes = resXProp.Value if portrait else resYProp.Value
  
  temp_view = app.createView('TemporaryBlenderer__VIEW')
  cam = app.findCamera()
  mtx = cam.Matrix
  temp_view.CameraMatrix = mtx
  temp_view.CameraFovy = fovProp.Value*yRes/xRes + 3
  app.useView('TemporaryBlenderer__VIEW')
  app.render()
  app.deleteView('TemporaryBlenderer__VIEW')
  renderBorder('foo')
  app.render()

def showBorderChangedEvent(arg):
  if showBorderProp.Value:
    app2.OnRender = renderBorder
    renderBorder('foo')
  else:
    app2.OnRender = None
    renderBorder('foo')
  app.render()


def renderBorder(foo):
  portrait = portraitProp.Value == 'Portrait'
  xRes = resYProp.Value if portrait else resXProp.Value
  yRes = resXProp.Value if portrait else resYProp.Value
  red = app.findMaterial('red')
  if showBorderProp.Value:
    cam = app.findCamera()
    cont = sim.World.UserGeometry
    cont.clear()
    lset = cont.createGeometrySet(VC_COMPACTLINESET)
    dist_offset = 10.0
    for i in range(5):
      horizontal = math.tan(fovProp.Value*DEG2RAD/2)*dist_offset
      vertical = float(yRes)/float(xRes)*horizontal
      v1 = vcVector.new(horizontal,-vertical,0)
      v2 = vcVector.new(horizontal,vertical,0)
      v3 = vcVector.new(-horizontal,vertical,0)
      v4 = vcVector.new(-horizontal,-vertical,0)
      v5 = vcVector.new(horizontal,-vertical,0)
      lines = [v1,v2,v3,v4,v5]
      mtx = cam.Matrix
      mtx.invert()
      mtx.translateRel(0,0,-dist_offset)
      lines = [mtx*x for x in lines]
      lset.addLine(lines)
      dist_offset *= 10.0
    lset.Material = red
  else:
    cont = sim.World.UserGeometry
    cont.clear()


def envtypeChanged(arg):
  if arg.Value == 'Image':
    envimageProp.IsVisible = True
    envcolorProp.IsVisible = False
  else:
    envimageProp.IsVisible = False
    envcolorProp.IsVisible = True

def floortypeChanged(arg):
  thefolder, commandfilename = os.path.split(getCommandPath()[8:])
  asphalt_uri = os.path.join('file:///'+thefolder, 'Textures', 'AsphaltOld.jpg')
  asphaltnormal_uri = os.path.join('file:///'+thefolder, 'Textures', 'AsphaltNormal.jpg')
  alu_uri = os.path.join('file:///'+thefolder, 'Textures', 'BrushedAlu.jpg')
  alunormal_uri = os.path.join('file:///'+thefolder, 'Textures', 'BrushedAluNormal.jpg')
  concretelight_uri = os.path.join('file:///'+thefolder, 'Textures', 'ConcreteLight.jpg')
  concretewhite_uri = os.path.join('file:///'+thefolder, 'Textures', 'ConcreteWhite.jpg')
  concretedark_uri = os.path.join('file:///'+thefolder, 'Textures', 'ConcreteDark.jpg')
  concretenormal_uri = os.path.join('file:///'+thefolder, 'Textures', 'ConcreteNormal.jpg')
  brownslate_uri = os.path.join('file:///'+thefolder, 'Textures', 'FlooringSlateBrown.jpg')
  greyslate_uri = os.path.join('file:///'+thefolder, 'Textures', 'FlooringSlateGrey.jpg')
  lightslate_uri = os.path.join('file:///'+thefolder, 'Textures', 'FlooringSlateLight.jpg')
  slatenormal_uri = os.path.join('file:///'+thefolder, 'Textures', 'FlooringSlateNormal.jpg')
  oak_uri = os.path.join('file:///'+thefolder, 'Textures', 'WoodOakTexture.jpg')
  oaknormal_uri = os.path.join('file:///'+thefolder, 'Textures', 'WoodOakNormal.jpg')
  floortextures = {
  'ConcreteLight':(concretelight_uri,concretenormal_uri),
  'ConcreteWhite':(concretewhite_uri,concretenormal_uri),
  'ConcreteDark':(concretedark_uri,concretenormal_uri),
  'SlateBrown':(brownslate_uri,slatenormal_uri),
  'SlateGrey':(greyslate_uri,slatenormal_uri),
  'SlateLight':(lightslate_uri,slatenormal_uri),
  'Oak':(oak_uri,oaknormal_uri),
  'Asphalt':(asphalt_uri,asphaltnormal_uri),
  'Alu':(alu_uri,alunormal_uri),
  }
  floorcolorimageProp.Value = str(floortextures[arg.Value][0])
  floorbumpimageProp.Value = str(floortextures[arg.Value][1])



addState(None)

material_columns = ['Name',
'R', 'G', 'B',
'metallic',
'roughness',
'bumpiness',
'clearcoat',
'clearcoat_rouhness',
'opacity',
'custom_data',
'color_map',
'normal_map']


